@@ -39,6 +39,7 @@ class CacheRecordingExtractor(BinDatRecordingExtractor, RecordingExtractor):
 
         self.set_tmp_folder(tmp_folder)
         self.copy_channel_properties(recording)
+        self.copy_times(recording)
 
         if 'gain' in recording.get_shared_channel_property_names() and not return_scaled:
             self.set_channel_gains(recording.get_channel_gains())
@@ -94,8 +95,6 @@ class CacheRecordingExtractor(BinDatRecordingExtractor, RecordingExtractor):
 
         Returns
         -------
-        dump_dict: dict
-            Serialized dictionary
         include_properties: list or None
             List of properties to include in the dictionary
         include_features: list or None
@@ -176,8 +175,6 @@ class CacheSortingExtractor(NpzSortingExtractor, SortingExtractor):
 
         Returns
         -------
-        dump_dict: dict
-            Serialized dictionary
         include_properties: list or None
             List of properties to include in the dictionary
         include_features: list or None
