@@ -6,7 +6,6 @@ import ctypes
 
 try:
     import h5py
-
     HAVE_BIOCAM = True
 except ImportError:
     HAVE_BIOCAM = False
@@ -15,13 +14,14 @@ except ImportError:
 class BiocamRecordingExtractor(RecordingExtractor):
     extractor_name = 'BiocamRecording'
     has_default_locations = True
+    has_unscaled = False
     installed = HAVE_BIOCAM  # check at class level if installed or not
     is_writable = True
     mode = 'file'
     installation_mesg = "To use the BiocamRecordingExtractor install h5py: \n\n pip install h5py\n\n"  # error message when not installed
 
     def __init__(self, file_path, verbose=False, mea_pitch=42):
-        assert HAVE_BIOCAM, self.installation_mesg
+        assert self.installed, self.installation_mesg
         self._mea_pitch = mea_pitch
         self._recording_file = file_path
         self._rf, self._nFrames, self._samplingRate, self._nRecCh, self._chIndices, \
@@ -46,7 +46,7 @@ class BiocamRecordingExtractor(RecordingExtractor):
         return self._samplingRate
 
     @check_get_traces_args
-    def get_traces(self, channel_ids=None, start_frame=None, end_frame=None):
+    def get_traces(self, channel_ids=None, start_frame=None, end_frame=None, return_scaled=True):
         data = self._read_function(self._rf, start_frame, end_frame, self.get_num_channels())
         # transform to slice if possible
         if sorted(channel_ids) == channel_ids and np.all(np.diff(channel_ids) == 1):
@@ -65,7 +65,7 @@ class BiocamRecordingExtractor(RecordingExtractor):
         # DigitalValue = (AnalogValue - MVOffset)/ADCCountsToMV
         # we center at 2048
 
-        assert HAVE_BIOCAM, "To use the BiocamRecordingExtractor install h5py: \n\n pip install h5py\n\n"
+        assert HAVE_BIOCAM, BiocamRecordingExtractor.installation_mesg
         M = recording.get_num_channels()
         N = recording.get_num_frames()
         rf = h5py.File(save_path, 'w')
@@ -140,7 +140,7 @@ def openBiocamFile(filename, mea_pitch, verbose=False):
         read_function = readHDF5t_101_i
     else:
         raise RuntimeError("File format unknown.")
-    return (rf, nFrames, samplingRate, nRecCh, chIndices, file_format, signalInv, rawIndices, read_function)
+    return rf, nFrames, samplingRate, nRecCh, chIndices, file_format, signalInv, rawIndices, read_function
 
 
 def readHDF5t_100(rf, t0, t1, nch):
