@@ -2,3 +2,5 @@ from .plexonextractor import PlexonRecordingExtractor, PlexonSortingExtractor
 from .neuralynxextractor import NeuralynxRecordingExtractor, NeuralynxSortingExtractor
 from .mcsrawrecordingextractor import MCSRawRecordingExtractor
 from .blackrockextractor import BlackrockRecordingExtractor, BlackrockSortingExtractor
+from .axonaextractor import AxonaRecordingExtractor
+from .spikegadgetsextractor import SpikeGadgetsRecordingExtractor
