@@ -2,18 +2,19 @@ from spikeextractors import RecordingExtractor
 from spikeextractors import SortingExtractor
 from pathlib import Path
 import numpy as np
-from functools import wraps
-from spikeextractors.extraction_tools import check_get_traces_args, check_valid_unit_id
+from spikeextractors.extraction_tools import check_get_traces_args, check_get_unit_spike_train, check_get_ttl_args
 
-'''
+"""
 The NumpyExtractors can be constructed and used to encapsulate custom file formats and data structures which
 contain information about recordings or sorting results. NumpyExtractors are instantiated in-memory and function
 like any other Recording/SortingExtractor.
-'''
+"""
 
 class NumpyRecordingExtractor(RecordingExtractor):
-    extractor_name = 'NumpyRecordingExtractor'
+    extractor_name = 'NumpyRecording'
     is_writable = True
+    has_default_locations = False
+    has_unscaled = False
 
     def __init__(self, timeseries, sampling_frequency, geom=None):
         RecordingExtractor.__init__(self)
@@ -38,6 +39,16 @@ class NumpyRecordingExtractor(RecordingExtractor):
         if geom is not None:
             self.set_channel_locations(self._geom)
 
+        self._ttl_frames = None
+        self._ttl_states = None
+
+    def set_ttls(self, ttl_frames, ttl_states=None):
+        self._ttl_frames = ttl_frames.astype('int64')
+        if ttl_states is not None:
+            self._ttl_states = ttl_states.astype('int64')
+        else:
+            self._ttl_states = np.ones_like(ttl_frames, dtype='int64')
+
     def get_channel_ids(self):
         return list(range(self._timeseries.shape[0]))
 
@@ -48,10 +59,19 @@ class NumpyRecordingExtractor(RecordingExtractor):
         return self._sampling_frequency
 
     @check_get_traces_args
-    def get_traces(self, channel_ids=None, start_frame=None, end_frame=None):
+    def get_traces(self, channel_ids=None, start_frame=None, end_frame=None, return_scaled=True):
         recordings = self._timeseries[:, start_frame:end_frame][channel_ids, :]
         return recordings
 
+    @check_get_ttl_args
+    def get_ttl_events(self, start_frame=None, end_frame=None, channel_id=0):
+        if self._ttl_frames is not None and self._ttl_states is not None:
+            ttl_idxs = np.where((self._ttl_frames >= start_frame) & (self._ttl_frames < end_frame))[0]
+            return self._ttl_frames[ttl_idxs], self._ttl_states[ttl_idxs]
+        else:
+            print("TTL frames have not been added to the extractor. You can add them with the `set_ttls()1 function")
+            return None, None
+
     @staticmethod
     def write_recording(recording, save_path):
         save_path = Path(save_path)
@@ -59,7 +79,7 @@ class NumpyRecordingExtractor(RecordingExtractor):
 
 
 class NumpySortingExtractor(SortingExtractor):
-    extractor_name = 'NumpySortingExtractor'
+    extractor_name = 'NumpySorting'
     is_writable = False
 
     def __init__(self):
@@ -68,7 +88,7 @@ class NumpySortingExtractor(SortingExtractor):
         self.is_dumpable = False
 
     def load_from_extractor(self, sorting, copy_unit_properties=False, copy_unit_spike_features=False):
-        '''This function loads the information from a SortingExtractor into this extractor.
+        """This function loads the information from a SortingExtractor into this extractor.
 
         Parameters
         ----------
@@ -78,7 +98,7 @@ class NumpySortingExtractor(SortingExtractor):
             If True, the unit_properties will be copied from the given SortingExtractor to this extractor.
         copy_unit_spike_features: bool
             If True, the unit_spike_features will be copied from the given SortingExtractor to this extractor.
-        '''
+        """
         ids = sorting.get_unit_ids()
         for id in ids:
             self.add_unit(id, sorting.get_unit_spike_train(id))
@@ -93,7 +113,7 @@ class NumpySortingExtractor(SortingExtractor):
         self._sampling_frequency = sampling_frequency
 
     def set_times_labels(self, times, labels):
-        '''This function takes in an array of spike times (in frames) and an array of spike labels and adds all the
+        """This function takes in an array of spike times (in frames) and an array of spike labels and adds all the
         unit information in these lists into the extractor.
 
         Parameters
@@ -102,14 +122,14 @@ class NumpySortingExtractor(SortingExtractor):
             An array of spike times (in frames).
         labels: np.array
             An array of spike labels corresponding to the given times.
-        '''
+        """
         units = np.sort(np.unique(labels))
         for unit in units:
             times0 = times[np.where(labels == unit)[0]]
             self.add_unit(unit_id=int(unit), times=times0)
 
     def add_unit(self, unit_id, times):
-        '''This function adds a new unit with the given spike times.
+        """This function adds a new unit with the given spike times.
 
         Parameters
         ----------
@@ -117,19 +137,14 @@ class NumpySortingExtractor(SortingExtractor):
             The unit_id of the unit to be added.
         times: np.array
             An array of spike times (in frames).
-        '''
+        """
         self._units[unit_id] = dict(times=times)
 
     def get_unit_ids(self):
         return list(self._units.keys())
 
-    @check_valid_unit_id
+    @check_get_unit_spike_train
     def get_unit_spike_train(self, unit_id, start_frame=None, end_frame=None):
-        start_frame, end_frame = self._cast_start_end_frame(start_frame, end_frame)
-        if start_frame is None:
-            start_frame = 0
-        if end_frame is None:
-            end_frame = np.Inf
         times = self._units[unit_id]['times']
         inds = np.where((start_frame <= times) & (times < end_frame))[0]
         return np.rint(times[inds]).astype(int)
