@@ -1,28 +1,34 @@
 import os
 from pathlib import Path
 import numpy as np
-from spikeextractors import SortingExtractor
+from spikeextractors import RecordingExtractor, SortingExtractor
 from spikeextractors.extractors.bindatrecordingextractor import BinDatRecordingExtractor
-from spikeextractors.extraction_tools import save_to_probe_file, load_probe_file, check_valid_unit_id
+from spikeextractors.extraction_tools import save_to_probe_file, load_probe_file, check_get_unit_spike_train
 
 try:
     import hybridizer.io as sbio
     import hybridizer.probes as sbprb
+    import yaml
+
     HAVE_SBEX = True
 except ImportError:
     HAVE_SBEX = False
 
 
-class SHYBRIDRecordingExtractor(BinDatRecordingExtractor):
+class SHYBRIDRecordingExtractor(RecordingExtractor):
     extractor_name = 'SHYBRIDRecording'
     installed = HAVE_SBEX
+    has_default_locations = True
+    has_unscaled = False
     is_writable = True
     mode = 'file'
-    installation_mesg = "To use the SHYBRID extractors, install SHYBRID: \n\n pip install shybrid\n\n"
+    installation_mesg = "To use the SHYBRID extractors, install SHYBRID and pyyaml: " \
+                        "\n\n pip install shybrid pyyaml\n\n"
 
     def __init__(self, file_path):
         # load params file related to the given shybrid recording
-        assert HAVE_SBEX, self.installation_mesg
+        assert self.installed, self.installation_mesg
+        RecordingExtractor.__init__(self)
         params = sbio.get_params(file_path)['data']
 
         # create a shybrid probe object
@@ -38,21 +44,35 @@ class SHYBRIDRecordingExtractor(BinDatRecordingExtractor):
             time_axis = 0
 
         # piggyback on binary data recording extractor
-        BinDatRecordingExtractor.__init__(self,
-                                          file_path,
-                                          params['fs'],
-                                          nb_channels,
-                                          params['dtype'],
-                                          time_axis=time_axis)
+        recording = BinDatRecordingExtractor(
+            file_path,
+            params['fs'],
+            nb_channels,
+            params['dtype'],
+            time_axis=time_axis)
+        
+        # load probe file
+        self._recording = load_probe_file(recording, params['probe'])
         self._kwargs = {'file_path': str(Path(file_path).absolute())}
-        self = load_probe_file(self, params['probe'])
 
+    def get_channel_ids(self):
+        return self._recording.get_channel_ids()
+
+    def get_num_frames(self):
+        return self._recording.get_num_frames()
+
+    def get_sampling_frequency(self):
+        return self._recording.get_sampling_frequency()
+
+    def get_traces(self, channel_ids=None, start_frame=None, end_frame=None, return_scaled=True):
+        return self._recording.get_traces(channel_ids=channel_ids, start_frame=start_frame, end_frame=end_frame,
+                                          return_scaled=return_scaled)
 
     @staticmethod
-    def write_recording(recording, save_path, initial_sorting_fn, dtype='float32'):
+    def write_recording(recording, save_path, initial_sorting_fn, dtype='float32', **write_binary_kwargs):
         """ Convert and save the recording extractor to SHYBRID format
 
-        parameters
+        Parameters
         ----------
         recording: RecordingExtractor
             The recording extractor to be converted and saved
@@ -63,6 +83,7 @@ class SHYBRIDRecordingExtractor(BinDatRecordingExtractor):
             using write_sorting static method from the SHYBRIDSortingExtractor)
         dtype: dtype
             Type of the saved data. Default float32.
+        **write_binary_kwargs: keyword arguments for write_to_binary_dat_format() function
         """
         assert HAVE_SBEX, SHYBRIDRecordingExtractor.installation_mesg
         RECORDING_NAME = 'recording.bin'
@@ -76,34 +97,35 @@ class SHYBRIDRecordingExtractor(BinDatRecordingExtractor):
 
         # write recording
         recording_fn = os.path.join(save_path, RECORDING_NAME)
-        BinDatRecordingExtractor.write_recording(recording, recording_fn,
-                                                 time_axis=0, dtype=dtype)
+        recording.write_to_binary_dat_format(save_path=recording_fn,
+                                             time_axis=0, dtype=dtype,
+                                             **write_binary_kwargs)
 
         # write probe file
         probe_fn = os.path.join(save_path, PROBE_NAME)
         save_to_probe_file(recording, probe_fn)
 
         # create parameters file
-        parameters = params_template.format(initial_sorting_fn=initial_sorting_fn,
-                                            data_type=dtype,
-                                            sampling_frequency=str(recording.get_sampling_frequency()),
-                                            byte_ordering='F',
-                                            probe_fn=probe_fn)
+        parameters = dict(clusters=initial_sorting_fn,
+                          data=dict(dtype=dtype,
+                                    fs=str(recording.get_sampling_frequency()),
+                                    order='F',
+                                    probe=probe_fn))
 
         # write parameters file
         parameters_fn = os.path.join(save_path, PARAMETERS_NAME)
         with open(parameters_fn, 'w') as fp:
-            fp.write(parameters)
+            yaml.dump(parameters, fp)
 
 
 class SHYBRIDSortingExtractor(SortingExtractor):
-    extractor_name = 'SHYBRIDSortingExtractor'
+    extractor_name = 'SHYBRIDSorting'
     installed = HAVE_SBEX
     is_writable = True
     installation_mesg = "To use the SHYBRID extractors, install SHYBRID: \n\n pip install shybrid\n\n"
 
     def __init__(self, file_path, delimiter=','):
-        assert HAVE_SBEX, self.installation_mesg
+        assert self.installed, self.installation_mesg
         SortingExtractor.__init__(self)
 
         if os.path.isfile(file_path):
@@ -116,16 +138,9 @@ class SHYBRIDSortingExtractor(SortingExtractor):
     def get_unit_ids(self):
         return self._spike_clusters.keys()
 
-    @check_valid_unit_id
+    @check_get_unit_spike_train
     def get_unit_spike_train(self, unit_id, start_frame=None, end_frame=None):
-        start_frame, end_frame = self._cast_start_end_frame(start_frame, end_frame)
         train = self._spike_clusters[unit_id].get_actual_spike_train().spikes
-
-        if start_frame is None:
-            start_frame = 0
-        if end_frame is None:
-            end_frame = np.Inf
-
         idxs = np.where((start_frame <= train) & (train < end_frame))
         return train[idxs]
 
@@ -144,8 +159,8 @@ class SHYBRIDSortingExtractor(SortingExtractor):
         dump = np.empty((0, 2))
 
         for unit_id in sorting.get_unit_ids():
-            spikes = sorting.get_unit_spike_train(unit_id)[:,np.newaxis]
-            expanded_id = (np.ones(spikes.size) * unit_id)[:,np.newaxis]
+            spikes = sorting.get_unit_spike_train(unit_id)[:, np.newaxis]
+            expanded_id = (np.ones(spikes.size) * unit_id)[:, np.newaxis]
             tmp_concat = np.concatenate((expanded_id, spikes), axis=1)
 
             dump = np.concatenate((dump, tmp_concat), axis=0)
@@ -159,12 +174,13 @@ class GeometryNotLoadedError(Exception):
     """
     pass
 
+
 params_template = \
-"""clusters:
-  csv: {initial_sorting_fn}
-data:
-  dtype: {data_type}
-  fs: {sampling_frequency}
-  order: {byte_ordering}
-  probe: {probe_fn}
-"""
+    """clusters:
+      csv: {initial_sorting_fn}
+    data:
+      dtype: {data_type}
+      fs: {sampling_frequency}
+      order: {byte_ordering}
+      probe: {probe_fn}
+    """
